'''
author: Yaodo
1 安居客房价爬取+生成图片
2 安居客把售罄的房源也算作新房，所以我觉得价格并不公允
3 我在爬到的结果中过滤了均价低于4000的房源（这样的房子是在太少了）
4 按套计价的也因为第三条原因被过滤了
'''

import requests,re
import matplotlib.pyplot as plt
from bs4 import BeautifulSoup
import pandas as pd
import seaborn as sns

headers = {
            'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/79.0.3945.88 Safari/537.36'
        }

#一个“城市”类，初试参数为城市代码和页数
class HousePriceCrawler(object):
    """docstring for HousePriceCrawler"""
    def __init__(self, city, page):
        self.city = city
        self.page = page
        self.raw_url = 'https://' + city +'.fang.anjuke.com/loupan/all/p'

    #获取每页的楼盘列表
    def get_house_list(self, i):
        url = self.raw_url + str(i) + '/'
        r = requests.get(url, headers = headers)
        soup = BeautifulSoup(r.text, 'lxml')
        results = soup.find_all('div', "key-list imglazyload")
        items = results[0].find_all('div', 'item-mod')
        return items

    #从每个楼盘中提取区县信息
    def get_location(self, item):
        district = re.compile('\[\xa0(.*?)\xa0')
        location = re.findall(district, str(item.find('span', 'list-map').string))[0]
        return location

    #获取面积（有些楼盘给的是总面积，所以这一项意义不大）
    def get_area(self, item):
        areapattern = re.compile('[0-9\-\.㎡]')
        area = ''.join(re.findall(areapattern, str(item.find('a', 'huxing').find_all('span')[-1].string)))
        return area

    #生成一条楼盘的信息（字典）
    def get_one(self, item):
        title = str(item.find('span', 'items-name').string)
        location = self.get_location(item)
        area = self.get_area(item)
        price = int(str(item.find('p', 'price').span.string))
        one = {
            'title': title,
            'location': location,
            'area': area,
            'price': price
        }
        return one

    #循环抓取每一页每一条楼盘信息，生成列表
    def Crawler(self):
        self.li = []
        for i in range(1, self.page+1):
            items = self.get_house_list(i)
            for item in items:
                try:
                    one = self.get_one(item)
                    self.li.append(one)
                except:
                    continue
            print('第{}页已完成…'.format(i))

    #根据列表生成DataFrame
    def get_frame(self):
        self.df = pd.DataFrame(self.li)
        self.df = self.df[self.df.price>4000]

    #汇总均价，用seaborn作图
    def draw_plot(self):
        self.Crawler()
        self.get_frame()
        price = self.df.groupby('location').mean()
        data = price.sort_values('price', ascending=False)
        f, ax = plt.subplots(figsize=(14,8))
        sns.barplot(x=data.index, y=data.price, data=data)
        plt.show()

#一个实例（淮安市）
HA = HousePriceCrawler('ha', 7)
HA.draw_plot()